import React from "react";
import { Box, Stack, Tooltip, styled } from "@mui/material";
import { STEPPER_STATES, describeState } from "./Stepper";

const StepIconWrapper = styled(
  ({ active, completed, error, disabled, innerRef, className, ...props }) => {
    let icon = null;
    if (active) {
      const ActiveIcon = STEPPER_STATES.active.filledIcon;
      icon = <ActiveIcon fontSize="small" />;
    }
    if (completed) {
      const CompletedIcon = STEPPER_STATES.completed.filledIcon;
      icon = <CompletedIcon fontSize="small" />;
    }
    if (error) {
      const ErrorIcon = STEPPER_STATES.error.filledIcon;
      icon = <ErrorIcon fontSize="small" />;
    }

    return (
      <div
        className={className}
        disabled={disabled}
        ref={innerRef}
        {...props}
      >
        {icon}
      </div>
    );
  }
)(({ active, completed, error, disabled }) => ({
  backgroundColor: "#ccc",
  zIndex: 1,
  color: "#fff",
  display: "flex",
  borderRadius: "50%",
  justifyContent: "center",
  alignItems: "center",
  ...(disabled && {
    borderStyle: "dashed",
    borderWidth: 2,
    width: 19,
    height: 19,
    background: "transparent",
    color: STEPPER_STATES.disabled.color.main
  }),
  ...(active && {
    backgroundColor: STEPPER_STATES.active.color.main
  }),
  ...(completed && {
    backgroundColor: `${STEPPER_STATES.completed.color.main}`
  }),
  ...(error && {
    backgroundColor: STEPPER_STATES.error.color.main
  })
}));

const Connector = styled(Box)(({ theme }) => ({
  width: 4,
  height: 1,
  borderBottom: `2px solid ${STEPPER_STATES.disabled.color.main}`,
  position: "relative",
  top: 9
}));

/**
 * Styled device-responsive Stepper.
 * @param {string[]} steps - List of step labels
 * @param {number} activeStep - Index of the current step, indexing from 0.
 * @param {boolean} isActiveStepFailed - true if current step is in an error state.
 * @returns Stepper
 */
export const InlineStepper = styled(
  ({ steps, activeStep, isActiveStepFailed, className }) => {
    const StepIcon = React.forwardRef(
      ({ active, completed, error, ...props }, ref) => {
        const disabled = !active && !completed && !error;

        return (
          <StepIconWrapper
            {...{ active, completed, error, disabled, innerRef: ref, ...props }}
          />
        );
      }
    );

    return (
      <Stack
        flexDirection="row"
        gap={0.1}
        divider={<Connector className="inline-stepper-connector" />}
        className={className}
        tabIndex={0}
        width="auto"
        aria-label={describeState({ steps, activeStep, isActiveStepFailed })}
      >
        {steps.map((label, index) => {
          const active = index === activeStep;
          const completed = index < activeStep;
          const error = isActiveStepFailed && active;

          return (
            <Tooltip
              title={`${label}`}
              key={`idx-${index}-label-${label}`}
            >
              <StepIcon {...{ active, completed, error }} />
            </Tooltip>
          );
        })}
      </Stack>
    );
  }
)({});
