import {
  Stepper as MuiStepper,
  Step,
  StepLabel,
  StepConnector,
  useMediaQuery,
  styled,
  stepConnectorClasses,
  Box
} from "@mui/material";
import {
  RotateRightOutlined,
  CheckCircleOutline,
  ErrorOutline
} from "@mui/icons-material";
import React from "react";
import { theme } from "../../../style/Theme";

export const STEPPER_STATES = {
  active: {
    color: theme.palette.status.progress,
    outlinedIcon: (props) => (
      <RotateRightOutlined
        style={{ color: theme.palette.status.progress.main }}
        {...props}
      />
    ),
    filledIcon: (props) => (
      <RotateRightOutlined
        style={{ color: theme.palette.essWhite.main }}
        {...props}
      />
    )
  },
  completed: {
    color: theme.palette.status.ok,
    outlinedIcon: (props) => (
      <CheckCircleOutline
        style={{ color: theme.palette.status.ok.main }}
        {...props}
      />
    ),
    filledIcon: (props) => (
      <CheckCircleOutline
        style={{ color: theme.palette.essWhite.main }}
        {...props}
      />
    )
  },
  error: {
    color: theme.palette.status.fail,
    outlinedIcon: (props) => (
      <ErrorOutline
        style={{ color: theme.palette.status.fail.main }}
        {...props}
      />
    ),
    filledIcon: (props) => (
      <ErrorOutline
        style={{ color: theme.palette.essWhite.main }}
        {...props}
      />
    )
  },
  disabled: {
    color: theme.palette.status.disabled
  }
};

export const describeState = ({ steps, activeStep, isActiveStepFailed }) => {
  const totalSteps = steps.length;
  const stepsRemaining = activeStep < totalSteps;
  const currentStep = activeStep <= 0 ? 0 : activeStep + 1;
  const completedSteps = currentStep - 1 >= 0 ? currentStep - 1 : 0;

  let ariaLabel = "no steps started";
  if (activeStep >= 0) {
    if (stepsRemaining) {
      ariaLabel = `${completedSteps} of ${totalSteps} steps completed, step ${
        activeStep + 1
      }: '${steps[activeStep]}' is ${
        isActiveStepFailed ? "in error" : "running"
      }`;
    } else {
      ariaLabel = "all steps completed";
    }
  }

  return ariaLabel;
};

const StepIconWrapper = styled(
  ({ active, completed, error, disabled, className }) => {
    let icon = null;
    if (active) {
      const ActiveIcon = STEPPER_STATES.active.filledIcon;
      icon = <ActiveIcon />;
    }
    if (completed) {
      const CompletedIcon = STEPPER_STATES.completed.filledIcon;
      icon = <CompletedIcon />;
    }
    if (error) {
      const ErrorIcon = STEPPER_STATES.error.filledIcon;
      icon = <ErrorIcon />;
    }

    return (
      <div
        className={className}
        disabled={disabled}
      >
        {icon}
      </div>
    );
  }
)(({ active, completed, error, disabled }) => ({
  backgroundColor: "#ccc",
  zIndex: 1,
  color: "#fff",
  width: 50,
  height: 50,
  display: "flex",
  borderRadius: "50%",
  justifyContent: "center",
  alignItems: "center",
  ...(disabled && {
    borderStyle: "dashed",
    borderWidth: 4,
    background: "transparent",
    color: STEPPER_STATES.disabled.color.main
  }),
  ...(active && {
    background: STEPPER_STATES.active.color.main,
    boxShadow: "0 4px 10px 0 rgba(0,0,0,0.25)"
  }),
  ...(completed && {
    background: STEPPER_STATES.completed.color.main
  }),
  ...(error && {
    background: STEPPER_STATES.error.color.main
  })
}));

const StyledStepConnector = styled(StepConnector)(() => ({
  [`&.${stepConnectorClasses.alternativeLabel}`]: {
    top: 22
  },
  [`& .${stepConnectorClasses.line}`]: {
    borderWidth: 4,
    [`&.${stepConnectorClasses.lineHorizontal}`]: {
      // Pull in the left and right so that the dashed
      // lines in disabled state don't interfere with
      // dashed border of (disabled) step icon
      marginLeft: 6,
      marginRight: 6,
      borderColor: STEPPER_STATES.disabled.color.main
    },
    [`&.${stepConnectorClasses.lineVertical}`]: {
      marginLeft: 11
    }
  },
  [`&.${stepConnectorClasses.disabled}`]: {
    [`& .${stepConnectorClasses.line}`]: {
      borderStyle: "dashed",
      borderWidth: 2,
      borderColor: STEPPER_STATES.disabled.color.main
    },
    [`& .${stepConnectorClasses.line}.${stepConnectorClasses.lineVertical}`]: {
      borderRight: 0,
      borderTop: 0,
      borderBottom: 0
    }
  }
}));

/**
 * Styled device-responsive Stepper.
 * @param {string[]} steps - List of step labels
 * @param {number} activeStep - Index of the current step, indexing from 0.
 * @param {boolean} isActiveStepFailed - true if current step is in an error state.
 * @returns Stepper
 */
export const Stepper = styled(
  ({ steps, activeStep, isActiveStepFailed, className }) => {
    const isDesktop = useMediaQuery(theme.breakpoints.up("sm"));

    const StepIcon = (props) => {
      const { active, completed, error } = props;
      const disabled = !active && !completed && !error;

      return (
        <StepIconWrapper
          {...{
            active,
            completed,
            error,
            disabled
          }}
        />
      );
    };

    return (
      <Box
        tabIndex={0}
        aria-label={describeState({ steps, activeStep, isActiveStepFailed })}
      >
        <MuiStepper
          activeStep={activeStep}
          orientation={isDesktop ? "horizontal" : "vertical"}
          connector={<StyledStepConnector />}
          alternativeLabel={isDesktop}
          className={className}
        >
          {steps.map((label, index) => {
            const labelProps = {};
            if (activeStep === index && isActiveStepFailed) {
              labelProps.error = true;
            }
            return (
              <Step key={`${index}-${label}`}>
                <StepLabel
                  StepIconComponent={StepIcon}
                  {...labelProps}
                >
                  {label}
                </StepLabel>
              </Step>
            );
          })}
        </MuiStepper>
      </Box>
    );
  }
)(theme);
