import { renderHook, waitFor } from "@testing-library/react";
import { useAPIMethod, useCreateAPIClient } from "./API";
import { useAsync } from "./Async";
import { sleep } from "./Async.test";

// These urls are intercepted by MSW
const options = {
  url: "https://petstore.com/spec",
  server: "https://petstore.com"
};

async function getClient() {
  const { result } = renderHook(() => useCreateAPIClient(options));
  await waitFor(() => expect(result.current.dataReady).toBe(true));
  return result.current.value;
}

describe("useCreateAPIClient", () => {
  it("builds API client from remote spec", async () => {
    const client = await getClient();
    expect(Object.keys(client.apis)).toEqual(["hosts", "echo", "auth"]);
    expect(client.spec.servers[0].url).toBe(options.server);
  });

  it("is possible to use the api", async () => {
    const client = await getClient();
    const { result } = renderHook(() =>
      useAsync({ fcn: client.apis.hosts.list_hosts })
    );
    await waitFor(() => expect(result.current.dataReady).toBe(true));

    const { hostList, totalCount } = result.current.value.obj;
    expect(totalCount).toBe(997);
    expect(hostList[0].owner).toBe("bcanter0");
  });
});

function simulateSlowClientCall(client) {
  return async (...args) => {
    await sleep(500);
    return client.apis.hosts.list_hosts(...args);
  };
}

describe("useAPIMethod", () => {
  it("correctly wraps the api method", async () => {
    const client = await getClient();
    const stub = jest.fn(client.apis.hosts.list_hosts);
    const { result } = renderHook(() => useAPIMethod({ fcn: stub }));
    await waitFor(() => expect(result.current.dataReady).toBe(true));
    expect(stub).toHaveBeenCalledTimes(1);

    const { hostList, totalCount } = result.current.value;
    expect(totalCount).toBe(997);
    expect(hostList[0].owner).toBe("bcanter0");
  });

  it("is possible to pass default parameters", async () => {
    const client = await getClient();
    const stub = jest.fn(client.apis.hosts.list_hosts);
    const params = { limit: 77 };
    const { result } = renderHook(() => useAPIMethod({ fcn: stub, params }));
    await waitFor(() => expect(result.current.dataReady).toBe(true));
    expect(stub).toHaveBeenCalledTimes(1);

    const { hostList, totalCount } = result.current.value;
    expect(totalCount).toBe(997);
    expect(hostList.length).toBe(77);
    expect(hostList[0].owner).toBe("bcanter0");
  });

  it("is possible to pass parameters and a request body at the call site", async () => {
    const client = await getClient();
    const stub = jest.fn(client.apis.echo.echo);
    const { result } = renderHook(() =>
      useAPIMethod({ fcn: stub, call: false })
    );
    await result.current.wrapper(
      { queryEcho: "hello" },
      { requestBody: { bodyEcho: "world" } }
    );
    await waitFor(() => expect(result.current.dataReady).toBe(true));
    expect(stub).toHaveBeenCalledTimes(1);

    const { queryEcho, bodyEcho } = result.current.value;
    expect(queryEcho).toBe("hello");
    expect(bodyEcho).toBe("world");
  });

  it("is possible to abort the api call directly", async () => {
    const client = await getClient();
    const stub = jest.fn(simulateSlowClientCall(client));
    const { result } = renderHook(() => useAPIMethod({ fcn: stub }));

    result.current.abort();

    await waitFor(() => expect(result.current.error.name).toBe("AbortError"));
    expect(result.current).toMatchObject({
      value: null,
      loading: false,
      dataReady: false
    });
    expect(stub).toHaveBeenCalledTimes(1);
  });

  it("aborts the api call on reentry", async () => {
    const client = await getClient();
    const stub = jest.fn(simulateSlowClientCall(client));
    const { result } = renderHook(() =>
      useAPIMethod({ fcn: stub, call: false })
    );

    // Should not have been called yet
    expect(stub).toHaveBeenCalledTimes(0);

    // When we call twice in a row quickly, then the previous call will be aborted
    // And we expect the wrapper to have been called again on reentry
    result.current.wrapper();
    result.current.wrapper();

    await waitFor(() => {
      expect(result.current.error.name).toBe("AbortError");
    });
    expect(result.current).toMatchObject({
      value: null,
      loading: false,
      dataReady: false
    });
    expect(stub).toHaveBeenCalledTimes(2);

    await waitFor(() =>
      expect(result.current).toMatchObject({ dataReady: true })
    );

    const { hostList, totalCount } = result.current.value;
    expect(totalCount).toBe(997);
    expect(hostList[0].owner).toBe("bcanter0");
    expect(result.current).toMatchObject({ error: null, loading: false });

    // Verify there haven't been any loops or
    // other oddities since receiving the result
    expect(stub).toHaveBeenCalledTimes(2);
  });
});
