import { useCallback, useEffect, useRef, useState } from "react";

/**
 * @external Promise
 * @see {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise Promise}
 */

/**
 * @typedef AsyncFunction
 * @returns Promise
 */

const defaultOnReentrance = () => false;
const defaultOnCleanup = () => {};

/**
 * @typedef UseAsyncHookProperties
 * @type {object}
 * @property {object} value Object returned by the resolved promise of `fcn`
 * @property {function} wrapper Callable wrapper for the `fcn`.
 * @property {boolean} loading Is `true` once `fcn` has been called, and becomes `false` once `fcn` resolves to data or an error.
 * @property {boolean} dataReady Is `false` until `fcn` resolves to data.
 * @property {object} error Result of `fcn` resolving to an error.
 */

/**
 *
 * @param {AsyncFunction} fcn Async function that will be invoked
 * @param {boolean} call If true, the function is invoked immediately on component mount, and runs onCleanup on component unmount.
 * @param {function} onCleanup function that is invoked during cleanup if call=true.
 * @param {object} init Initial value to return from `fcn` before promise has resolved.
 * @param {function} onReentrance function that will be invoked if `fcn` is called again before it has resolved.
 * @returns {UseAsyncHookProperties}
 */
export function useAsync({
  fcn,
  call = true,
  onCleanup = defaultOnCleanup,
  init = null,
  onReentrance = defaultOnReentrance
}) {
  const [value, setValue] = useState(init);
  const [loading, setLoading] = useState(false);
  const [dataReady, setDataReady] = useState(false);
  const [error, setError] = useState(null);
  const busyRef = useRef(false);

  const wrapper = useCallback(
    async (...args) => {
      if (busyRef.current) {
        if (onReentrance(...args)) {
          console.debug("re-entrance defined, returning...");
          return;
        }
      }
      busyRef.current = true;
      console.debug("entering first time");
      setLoading(true);
      setError(null);
      try {
        const result = await fcn(...args);
        console.debug("results received", result);
        setValue(result);
        setError(null);
        setDataReady(true);
      } catch (e) {
        setError(e);
        console.debug(`error during async, ${e?.name}:`, e);
      } finally {
        setLoading(false);
        busyRef.current = false;
      }
    },
    [fcn, onReentrance]
  );

  useEffect(() => {
    if (call) {
      wrapper();
    }

    return () => {
      onCleanup();
    };
  }, [wrapper, call, onCleanup]);

  return { value, wrapper, loading, dataReady, error };
}
