import { matchPath } from "react-router-dom";

// Utility to extract search params from the
// String representation of the request url
// (Swagger client's request is a fetch Request,
// so the url is a String and not a URL object)
function searchParams(req) {
  if (typeof req.url === "string") {
    return new URL(req.url).searchParams;
  } else {
    return req.url.searchParams;
  }
}

function getParameters(req, pattern) {
  const match = matchPath({ path: pattern }, new URL(req.url).pathname);
  return match?.params;
}

function withMockPagination({ req, data, key }) {
  const params = new URL(req.url).searchParams;
  const page = Number(params.get("page")) || 0;
  const limit = Number(params.get("limit")) || 99999;

  const startIndex = page * limit;
  const resultList = key ? data[key] : data;
  const pagedData = resultList.slice(startIndex, startIndex + limit);

  const dataKey = key ?? "data";

  const results = {
    limit,
    listSize: pagedData.length,
    pageNumber: page,
    totalCount: resultList.length,
    [dataKey]: pagedData,
    DEVELOPER_NOTE: "THIS IS MOCKED PAGING!"
  };

  return results;
}

// handler to return the OpenAPI spec
function spec(req) {
  const data = require("./fixtures/api/example-spec.json");
  return {
    body: data
  };
}

function get_host(req) {
  const data = require("./fixtures/api/hosts.json");
  const params = getParameters(req, "/hosts/:id");
  const body = data.find((x) => x.id === Number(params.id));
  if (body) {
    return { body };
  } else {
    return { status: 404 };
  }
}

function list_hosts(req) {
  const hostData = require("./fixtures/api/hosts.json");
  const {
    // page = 0,
    // limit = 10,
    query = ""
  } = Object.fromEntries(searchParams(req));
  const filteredData = query
    ? hostData.filter((h) => h.name.includes(query))
    : hostData;

  const { data, ...rest } = withMockPagination({ req, data: filteredData });
  return {
    body: {
      ...rest,
      hostList: data
    }
  };
}

function echo(req) {
  const { queryEcho } = Object.fromEntries(searchParams(req));
  const { bodyEcho } = req.body;
  return {
    body: { queryEcho, bodyEcho }
  };
}

function api_login(req) {
  if (req?.body?.username?.length > 0 && req?.body?.password?.length > 0) {
    return {
      body: { token: req.body.username }
    };
  }

  return { body: { message: "Bad user name or password" }, status: 401 };
}

// just to organize
const mockAPI = {
  spec: spec,
  hosts: {
    list_hosts,
    get_host
  },
  echo: {
    echo
  },
  auth: {
    api_login
  }
};

// Enough info to create handlers in cypress or MSW
const makeHandler = (method, matcher, requestHandler, wrapper, delay) => ({
  method,
  matcher,
  requestHandler: wrapper ? wrapper(requestHandler) : requestHandler,
  name: requestHandler.name, // use caution when changing api function names
  delay: delay
});

const queryPattern = "(\\?.*)?$";
const qRegExp = (pattern) => RegExp(pattern + queryPattern);

// Handlers for our whole API
export const apiHandlers = [
  // api spec
  makeHandler("GET", qRegExp(".*/spec"), mockAPI.spec),

  // hosts
  makeHandler(
    "GET",
    qRegExp(".*/hosts/[0-9]+"),
    mockAPI.hosts.get_host,
    null,
    2000
  ),
  makeHandler("GET", qRegExp(".*/hosts"), mockAPI.hosts.list_hosts),

  // echo
  makeHandler("POST", qRegExp(".*/echo"), mockAPI.echo.echo),

  // login
  makeHandler("POST", qRegExp(".*/api_login"), mockAPI.auth.api_login)
];
