/* eslint-disable no-console */
import React, { useContext, useEffect, useMemo } from "react";
import { apiOptions, apiContext } from "../utils/common-data";
import { Button, CircularProgress, Stack, Typography } from "@mui/material";
import { APIProvider, useAPIMethod } from "../../hooks/API";

export default {
  title: "hooks/useApiMethod"
};

const Harness = ({ children }) => (
  <APIProvider
    context={apiContext}
    options={apiOptions}
  >
    {children}
  </APIProvider>
);

const Row = ({ label, value }) => {
  return (
    <Stack
      flexDirection="row"
      gap={1}
    >
      <Typography>{label}:</Typography>
      <Typography>{value}</Typography>
    </Stack>
  );
};

const renderData = (data) => (data ? `${data.name} (id ${data.id})` : null);
const renderError = (e) => (e ? `${e?.name} ${e?.status} ${e?.message}` : null);

const Container = ({ hostId = 1, call = true, ...args }) => {
  if (call) {
    console.log("call=true; calling for API value immediately inside hook");
  } else {
    console.log(
      "call=false; calling and cleaning up API value in component via effect"
    );
  }

  const client = useContext(apiContext);

  const params = useMemo(
    () => ({
      host_id: hostId
    }),
    [hostId]
  );

  const {
    wrapper: getValue,
    value,
    loading,
    dataReady,
    error,
    abort
  } = useAPIMethod({
    fcn: client?.apis.hosts.get_host,
    params,
    call,
    unpacker: renderData
  });

  console.log({ value, loading, dataReady, error });

  useEffect(() => {
    if (!call) {
      console.log("Calling from effect");
      getValue();

      return () => {
        console.log("Cleaning up from effect");
        abort();
      };
    }
  }, [call, getValue, abort]);

  return (
    <Stack gap={1}>
      <Stack
        flexDirection="row"
        alignContent="center"
        gap={1}
      >
        <Row
          label="loading"
          value={`${loading}`}
        />
        {loading ? <CircularProgress size={24} /> : null}
      </Stack>
      <Row
        label="dataReady"
        value={`${dataReady}`}
      />
      <Row
        label="value"
        value={value}
      />
      <Row
        label="error"
        value={renderError(error)}
      />
      <Button
        variant="contained"
        onClick={() => getValue()}
        sx={{ width: "300px" }}
      >
        Call Again
      </Button>
      <Button
        variant="contained"
        onClick={() => abort()}
        sx={{ width: "300px" }}
      >
        Abort
      </Button>
    </Stack>
  );
};

const Template = (args) => {
  return (
    <Harness>
      <Container {...args} />
    </Harness>
  );
};

export const Default = (args) => <Template {...args} />;
Default.args = {
  hostId: 1
};

export const CallAndAbortManually = (args) => (
  <Template
    call={false}
    {...args}
  />
);
CallAndAbortManually.args = { ...Default.args };
